<?php

namespace Drupal\autotagger\Plugin\Autotagger;

use Drupal\autotagger\AutotaggerPluginBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\NodeInterface;
use Drupal\node\Entity\NodeType;
use Drupal\node\NodeTypeInterface;

/**
 * Plugin implementation of the autotagger.
 *
 * @Autotagger(
 *   id = "search_in_text_fields",
 *   label = @Translation("Search in Text fields"),
 *   description = @Translation("Searches in text fields and tags any appearence."),
 *   configurable = false
 * )
 */
class SearchInTextFields extends AutotaggerPluginBase {

  /**
   * {@inheritdoc}
   */
  public function addFormOptions(&$form, FormStateInterface $form_state, $form_id) {
    if ($form_id != 'node_type_add_form' && $form_id != 'node_type_edit_form') {
      return;
    }

    $entityManager = \Drupal::service('entity_field.manager');
    $source_fields = [];
    $destination_fields = [];

    $default_values = $form_state->getFormObject()->getEntity()->getThirdPartySetting('autotagger', 'search_in_text_fields');
    $form['autotagger'] = [
      '#type' => 'details',
      '#group' => 'additional_settings',
      '#title' => 'Autotagger : ' . $this->label(),
    ];

    $type = $form['type']['#default_value'];

    $fields = $entityManager->getFieldDefinitions('node', $type);

    /** @var \Drupal\field\Entity\FieldConfig $field */
    foreach ($fields as $field) {
      $field_type = $field->getType();

      // Text fields can be used as a source.
      $text_types = [
        'text',
        'string',
        'text_with_summary',
        'string_with_summary',
        'text_long',
        'string_long',
      ];

      if (in_array($field_type, $text_types)) {
        $source_fields[$field->getName()] = $field->getLabel();
      }

      // Text fields on node or media references can be used as a source.
      $target_types = ['node', 'media'];
      if ($field_type == 'entity_reference' && (in_array($field->getSettings()['target_type'], $target_types))) {
        $source_fields[$field->getName()] = t('Text fields on entities referenced by @label', ['@label' => $field->getLabel()])->render();
      }

      // Text fields on paragraphs can be used as a source.
      if ($field_type == 'entity_reference_revisions') {
        $source_fields[$field->getName()] = t('Text fields on paragraphs referenced by @label', ['@label' => $field->getLabel()])->render();
      }

      // Taxonomy references can be used as a destination.
      if ($field_type == 'entity_reference' && $field->getSettings()['target_type'] == 'taxonomy_term') {
        $destination_fields[$field->getName()] = $field->getLabel();
      }
    }

    $form['autotagger']['autotagger_source_field'] = [
      '#type' => 'checkboxes',
      '#title' => t('Source fields'),
      '#description' => t('Choose which fields to scan for terms. Checking an entity reference field will cause all text fields on the referenced entity to be scanned.'),
      '#default_value' => $default_values['autotagger_source_field'] ?? [],
      '#options' => $source_fields,
      '#access' => TRUE,
    ];

    $form['autotagger']['autotagger_destination_field'] = [
      '#type' => 'select',
      '#title' => t('Destination fields'),
      '#description' => t('Choose the taxonomy reference field to use. The vocabularies enabled on this field will provide the scanned for terms.'),
      '#default_value' => $default_values['autotagger_destination_field'] ?? '',
      '#options' => $destination_fields,
      '#access' => TRUE,
    ];

    $form['autotagger']['autotagger_tag_on_create_only'] = [
      '#type' => 'checkbox',
      '#title' => t('Only assign tags on initial node creation.'),
      '#default_value' => $default_values['autotagger_tag_on_create_only'] ?? '',
      '#access' => TRUE,
    ];

    $form['autotagger']['autotagger_rebuild_on_submit'] = [
      '#type' => 'checkboxes',
      '#title' => t('Assign tags on save.'),
      '#default_value' => $default_values['autotagger_rebuild_on_submit'] ?? '',
      '#options' => ['rebuild' => t('Rebuild on this form submission')],
      '#access' => TRUE,
    ];

    $form['actions']['submit']['#submit'][] = 'autotagger_form_submit_autotag_nodes';
    $form['#entity_builders'][] = [$this, 'entityBuilder'];
  }

  /**
   * {@inheritdoc}
   */
  public function entityPresave(NodeInterface $node) {
    // Determine if this node is set to tag only on node creation.
    $node_type = NodeType::load($node->bundle());
    $settings = $node_type->getThirdPartySetting('autotagger', 'search_in_text_fields');
    $create_only = $settings['autotagger_tag_on_create_only'];

    if ($node->isNew() || !$create_only) {
      // If the node is first being created, or the node is set to not only tag
      // on new, then go ahead and tag it.
      autotagger_tag_node($node);
    }
  }

  /**
   * Entity builder for the menu configuration entity.
   */
  public function entityBuilder($entity_type, NodeTypeInterface $node_type, array &$form, FormStateInterface $form_state) {
    $autotagger = [
      'autotagger_source_field' => $form_state->getValue('autotagger_source_field'),
      'autotagger_destination_field' => $form_state->getValue('autotagger_destination_field'),
      'autotagger_tag_on_create_only' => $form_state->getValue('autotagger_tag_on_create_only'),
      'autotagger_rebuild_on_submit' => $form_state->getValue('autotagger_rebuild_on_submit'),
    ];
    $node_type->setThirdPartySetting('autotagger', 'search_in_text_fields', $autotagger);
  }

}
